﻿using Connected.Annotations;
using Connected.Utilities;
using Microsoft.AspNetCore.Components;

namespace Connected.Components;

public partial class NavGroup : UIComponent
{
	#region Variables
	private bool _expanded;
	#endregion

	#region Events
	protected void ExpandedToggle()
	{
		_expanded = !Expanded;
		ExpandedChanged.InvokeAsync(_expanded);
	}
	[Parameter] public EventCallback<bool> ExpandedChanged { get; set; }
	#endregion

	#region Content
	[Parameter]
	[Category(CategoryTypes.NavMenu.Behavior)]
	public string Title { get; set; }

	/// <summary>
	/// Glyph to use if set.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Behavior)]
	public string Icon { get; set; }

	/// <summary>
	/// The color of the icon. It supports the theme colors, default value uses the themes drawer icon color.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Appearance)]
	public ThemeColor IconColor { get; set; } = ThemeColor.Default;

	/// <summary>
	/// If set, overrides the default expand icon.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Appearance)]
	public string ExpandIcon { get; set; } = @Icons.Filled.ArrowDropDown;
	#endregion

	#region Styling
	protected string Classname =>
	new CssBuilder("nav-group")
	  .AddClass(AdditionalClassList)
	  .AddClass($"nav-group-disabled", Disabled)
	  .Build();

	protected string ButtonClassname =>
	new CssBuilder("nav-link")
	  .AddClass($"ripple", !DisableRipple)
	  .AddClass("expanded", Expanded)
	  .Build();

	protected string IconClassname =>
	new CssBuilder("nav-link-icon")
	  .AddClass($"nav-link-icon-default", IconColor == ThemeColor.Default)
	  .Build();

	protected string ExpandIconClassname =>
	new CssBuilder("nav-link-expand-icon")
	  .AddClass($"transform", Expanded && !Disabled)
	  .AddClass($"transform-disabled", Expanded && Disabled)
	  .Build();

	/// <summary>
	/// If true, hides expand-icon at the end of the NavGroup.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Appearance)]
	public bool HideExpandIcon { get; set; }

	/// <summary>
	/// Explicitly sets the height for the Collapse element to override the css default.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Appearance)]
	public int? MaxHeight { get; set; }

	[Parameter]
	[Category(CategoryTypes.NavMenu.Behavior)]
	public RenderFragment ChildContent { get; set; }
	#endregion

	#region Behavior
	/// <summary>
	/// If true, the button will be disabled.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Behavior)]
	public bool Disabled { get; set; }

	/// <summary>
	/// If true, disables ripple effect.
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Appearance)]
	public bool DisableRipple { get; set; }


	/// <summary>
	/// If true, expands the nav group, otherwise collapse it. 
	/// Two-way bindable
	/// </summary>
	[Parameter]
	[Category(CategoryTypes.NavMenu.Behavior)]
	public bool Expanded
	{
		get => _expanded;
		set
		{
			if (_expanded == value)
				return;

			_expanded = value;
			ExpandedChanged.InvokeAsync(_expanded);
		}
	}
	#endregion

}
